@@ -36,6 +36,8 @@
 #include <locale.h>
 #endif
 
+extern gchar *gda_lang_locale;
+
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-decl.h>
 #include <libgda/gda-enums.h>
@@ -44,6 +46,7 @@
 #include <libgda/gda-data-model-extra.h>
 #include <libgda/gda-data-access-wrapper.h>
 #include <libgda/gda-data-model-iter.h>
+#include <libgda/gda-data-model-iter-extra.h>
 #include <libgda/gda-holder.h>
 #include <libgda/gda-set.h>
 #include <libgda/gda-data-model-private.h> /* For gda_data_model_add_data_from_xml_node() */
@@ -153,12 +156,12 @@ static void gda_data_model_import_get_property (GObject *object,
 						GParamSpec *pspec);
 
 /* GdaDataModel interface */
-static void                 gda_data_model_import_data_model_init (GdaDataModelClass *iface);
+static void                 gda_data_model_import_data_model_init (GdaDataModelIface *iface);
 static gint                 gda_data_model_import_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_import_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_import_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_model_import_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row);
 static GdaDataModelIter    *gda_data_model_import_create_iter      (GdaDataModel *model);
 static gboolean             gda_data_model_import_iter_next       (GdaDataModel *model, GdaDataModelIter *iter); 
@@ -221,24 +224,25 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 	object_class->set_property = gda_data_model_import_set_property;
         object_class->get_property = gda_data_model_import_get_property;
 	g_object_class_install_property (object_class, PROP_RANDOM_ACCESS,
-                                         g_param_spec_boolean ("random_access", "Features random access", NULL,
+                                         g_param_spec_boolean ("random-access", NULL, "Random access to the data model "
+							       "is possible",
 							       FALSE,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_FILENAME,
-                                         g_param_spec_string ("filename", "File to import", NULL, NULL,
+                                         g_param_spec_string ("filename", NULL, "File to import", NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_DATA_STRING,
-                                         g_param_spec_string ("data_string", "String to import", NULL, NULL,
+                                         g_param_spec_string ("data-string", NULL, "String to import", NULL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_XML_NODE,
-                                         g_param_spec_pointer ("xml_node", "XML node to import from", NULL,
+                                         g_param_spec_pointer ("xml-node", NULL, "XML node to import from",
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OPTIONS,
-                                         g_param_spec_object ("options", "Options to configure the import", NULL,
+                                         g_param_spec_object ("options", NULL, "Options to configure the import",
                                                                GDA_TYPE_SET,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
@@ -249,7 +253,7 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 }
 
 static void
-gda_data_model_import_data_model_init (GdaDataModelClass *iface)
+gda_data_model_import_data_model_init (GdaDataModelIface *iface)
 {
 	iface->i_get_n_rows = gda_data_model_import_get_n_rows;
 	iface->i_get_n_columns = gda_data_model_import_get_n_columns;
@@ -264,6 +268,7 @@ gda_data_model_import_data_model_init (GdaDataModelClass *iface)
 	iface->i_iter_prev = gda_data_model_import_iter_prev;
 
 	iface->i_set_value_at = NULL;
+	iface->i_iter_set_value = NULL;
 	iface->i_set_values = NULL;
         iface->i_append_values = NULL;
 	iface->i_append_row = NULL;
@@ -678,7 +683,7 @@ gda_data_model_import_get_property (GObject *object,
  *         <listitem><para>SEPARATOR (string): specifies the CSV separator (comma as default)</para></listitem>
  *         <listitem><para>QUOTE (string): specifies the character used to as quote park (double quote as default)</para></listitem>
  *         <listitem><para>TITLE_AS_FIRST_LINE (boolean): consider that the first line of the file contains columns' titles</para></listitem>
- *         <listitem><para>DBMS_TYPE_&lt;column number&gt; (string): specifies the type of value expected in column &lt;column number&gt;</para></listitem>
+ *         <listitem><para>G_TYPE_&lt;column number&gt; (GType): specifies the type of value expected in column &lt;column number&gt;</para></listitem>
  *      </itemizedlist>
  *   </para></listitem>
  *   <listitem><para>Other formats: no option</para></listitem>
@@ -694,7 +699,7 @@ gda_data_model_import_new_file   (const gchar *filename, gboolean random_access,
 	g_return_val_if_fail (filename, NULL);
 	
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "random_access", random_access,
+			      "random-access", random_access,
 			      "options", options,
 			      "filename", filename, NULL);
 
@@ -720,9 +725,9 @@ gda_data_model_import_new_mem (const gchar *data, gboolean random_access, GdaSet
 	GdaDataModelImport *model;
 
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "random_access", random_access,
+			      "random-access", random_access,
 			      "options", options,
-			      "data_string", data, NULL);
+			      "data-string", data, NULL);
 
 	return GDA_DATA_MODEL (model);
 }
@@ -742,7 +747,7 @@ gda_data_model_import_new_xml_node (xmlNodePtr node)
 	GdaDataModelImport *model;
 
 	model = g_object_new (GDA_TYPE_DATA_MODEL_IMPORT,
-			      "xml_node", node, NULL);
+			      "xml-node", node, NULL);
 
 	return GDA_DATA_MODEL (model);
 }
@@ -816,36 +821,28 @@ init_csv_import (GdaDataModelImport *model)
 		if (!str) 
 			str = g_strdup_printf ("column_%d", col);
 		gda_column_set_name (column, str);
-		gda_column_set_title (column, str);
-		gda_column_set_caption (column, str);
+		gda_column_set_description (column, str);
 		g_free (str);			
 		
 		gda_column_set_g_type (column, G_TYPE_STRING);
 		if (model->priv->options) {
 			gchar *pname;
-			const gchar *dbms_t;
 			const GValue *value;
 			
-			pname = g_strdup_printf ("GDA_TYPE_%d", col);
+			pname = g_strdup_printf ("G_TYPE_%d", col);
 			value = gda_set_get_holder_value (model->priv->options, pname);
 			if (value && !gda_value_is_null ((GValue *) value)) {
-				if (!gda_value_isa ((GValue *) value, G_TYPE_ULONG))
-					g_warning (_("The '%s' parameter must hold a "
-						     "gda type value, ignored."), pname);
+				if (!gda_value_isa ((GValue *) value, G_TYPE_GTYPE))
+					g_warning (_("The '%s' option must hold a "
+						     "GType value, ignored."), pname);
 				else {
 					GType gtype;
 					
-					gtype = g_value_get_ulong ((GValue *) value);
+					gtype = g_value_get_gtype ((GValue *) value);
 					gda_column_set_g_type (column, gtype);
 				}
 			}
 			g_free (pname);
-			
-			pname = g_strdup_printf ("DBMS_TYPE_%d", col);
-			dbms_t = find_option_as_string (model, pname);
-			if (dbms_t) 
-				gda_column_set_dbms_type (column, dbms_t);
-			g_free (pname);
 		}
 	}
 
@@ -1126,6 +1123,7 @@ init_xml_import (GdaDataModelImport *model)
 			g_free (str);
 			xmlFreeTextReader (reader);
 			model->priv->extract.xml.reader = NULL;
+			return;
 		}
 		node = xmlTextReaderCurrentNode (reader);
 		
@@ -1228,18 +1226,11 @@ init_xml_import (GdaDataModelImport *model)
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns, column);
 			g_object_set (G_OBJECT (column), "id", spec->id, NULL);
-			gda_column_set_title (column, (gchar*)spec->title);
+			gda_column_set_description (column, (gchar*)spec->title);
 			gda_column_set_name (column, (gchar*)spec->name);
-			gda_column_set_defined_size (column, spec->size);
-			gda_column_set_caption (column, (gchar*)spec->caption);
 			gda_column_set_dbms_type (column, (gchar*)spec->dbms_type);
-			gda_column_set_scale (column, spec->scale);
 			gda_column_set_g_type (column, spec->gdatype);
 			gda_column_set_allow_null (column, spec->nullok);
-			gda_column_set_primary_key (column, spec->pkey);
-			gda_column_set_unique_key (column, spec->unique);
-			gda_column_set_table (column, (gchar*)spec->table);
-			gda_column_set_references (column, (gchar*)spec->ref);
 			
 			list = g_slist_next (list);
 			pos++;
@@ -1294,18 +1285,12 @@ xml_fetch_next_row (GdaDataModelImport *model)
 	const xmlChar *name;
 	gint ret;
 
-	const gchar *lang = NULL;
+	const gchar *lang = gda_lang_locale;
 
 	GSList *columns = model->priv->columns;
 	GdaColumn *last_column = NULL;
 	GSList *values = NULL;
 
-#ifdef HAVE_LC_MESSAGES
-	lang = setlocale (LC_MESSAGES, NULL);
-#else
-	lang = setlocale (LC_CTYPE, NULL);
-#endif
-
 	if (model->priv->cursor_values) {
 		g_slist_foreach (model->priv->cursor_values, (GFunc) gda_value_free, NULL);
 		g_slist_free (model->priv->cursor_values);
@@ -1572,18 +1557,11 @@ init_node_import (GdaDataModelImport *model)
 		spec = (XmlColumnSpec *)(list->data);
 		column = gda_data_model_describe_column (ramodel, pos);
 		g_object_set (G_OBJECT (column), "id", spec->id, NULL);
-		gda_column_set_title (column, (gchar*)spec->title);
+		gda_column_set_description (column, (gchar*)spec->title);
 		gda_column_set_name (column, (gchar*)spec->name);
-		gda_column_set_defined_size (column, spec->size);
-		gda_column_set_caption (column, (gchar*)spec->caption);
 		gda_column_set_dbms_type (column, (gchar*)spec->dbms_type);
-		gda_column_set_scale (column, spec->scale);
 		gda_column_set_g_type (column, spec->gdatype);
 		gda_column_set_allow_null (column, spec->nullok);
-		gda_column_set_primary_key (column, spec->pkey);
-		gda_column_set_unique_key (column, spec->unique);
-		gda_column_set_table (column, (gchar*)spec->table);
-		gda_column_set_references (column, (gchar*)spec->ref);
 
 		model->priv->columns = g_slist_prepend (model->priv->columns, gda_column_copy (column));
 
@@ -1604,7 +1582,7 @@ add_error (GdaDataModelImport *model, const gchar *err)
 {
 	GError *error = NULL;
 
-	g_set_error (&error, 0, 0, err);
+	g_set_error (&error, 0, 0, "%s", err);
 	model->priv->errors = g_slist_append (model->priv->errors, error);
 }
 
@@ -1715,7 +1693,7 @@ gda_data_model_import_get_access_flags (GdaDataModel *model)
 }
 
 static const GValue *
-gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataModelImport *imodel;
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_IMPORT (model), NULL);
@@ -1724,10 +1702,13 @@ gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
 
 	if (imodel->priv->random_access_model)
 		/* if there is a random access model, then use it */
-		return gda_data_model_get_value_at (imodel->priv->random_access_model, col, row);
-	else
+		return gda_data_model_get_value_at (imodel->priv->random_access_model, col, row, error);
+	else {
 		/* otherwise, bail out */
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			      "%s", _("Data model does not support random access"));
 		return NULL;
+	}
 }
 
 static GdaValueAttribute
@@ -1770,7 +1751,7 @@ gda_data_model_import_create_iter (GdaDataModel *model)
 	}
 	else
 		iter = (GdaDataModelIter *) g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
-							  "data_model", model, NULL);
+							  "data-model", model, NULL);
 	return iter;
 }
 
@@ -1825,7 +1806,7 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 
 	/* if there is a random access model, then use it */
 	if (imodel->priv->format == FORMAT_XML_NODE)
-		return gda_data_model_move_iter_next_default (model, iter);
+		return gda_data_model_iter_move_next_default (model, iter);
 
 	/* fetch the next row if necessary */
 	switch (imodel->priv->format) {
@@ -1862,24 +1843,30 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 		GSList *plist;
 		GSList *vlist;
 		gboolean update_model;
+		gboolean allok = TRUE;
 		
-		g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-		g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-		plist = ((GdaSet *) iter)->holders;
-		vlist = next_values;
-		while (plist && vlist) {
-			gda_holder_set_value (GDA_HOLDER (plist->data), 
-					      (GValue *) vlist->data);
-			plist = g_slist_next (plist);
-			vlist = g_slist_next (vlist);
+		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+		for (plist = ((GdaSet *) iter)->holders, vlist = next_values;
+		     plist && vlist;
+		     plist = plist->next, vlist = vlist->next) {
+			GError *lerror = NULL;
+			if (! gda_holder_set_value (GDA_HOLDER (plist->data), 
+						    (GValue *) vlist->data, &lerror)) {
+				gchar *tmp;
+				tmp = g_strdup_printf (_("Could not set iterator's value: %s"),
+						       lerror && lerror->message ? lerror->message : _("No detail"));
+				add_error (imodel, tmp);
+				g_free (tmp);
+				allok = FALSE;
+			}
 		}
 		if (plist || vlist) {
+			allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				while (plist) {
-					gda_holder_set_value (GDA_HOLDER (plist->data), NULL);
-					plist = g_slist_next (plist);
-				}
+				for (; plist; plist = plist->next)
+					gda_holder_force_invalid (GDA_HOLDER (plist->data));
 			}
 			else
 				add_error_too_many_values (imodel);
@@ -1890,12 +1877,12 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 			imodel->priv->iter_row = 0;
 
 		g_object_set (G_OBJECT (iter), "current-row", imodel->priv->iter_row, 
-			      "update_model", update_model, NULL);
+			      "update-model", update_model, NULL);
 
-		return TRUE;
+		return allok;
 	}
 	else {
-		g_signal_emit_by_name (iter, "end_of_data");
+		g_signal_emit_by_name (iter, "end-of-data");
 		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
 		return FALSE;
 	}
@@ -1915,7 +1902,7 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 
 	/* if there is a random access model, then use it */
 	if (imodel->priv->format == FORMAT_XML_NODE)
-		return gda_data_model_move_iter_prev_default (model, iter);
+		return gda_data_model_iter_move_prev_default (model, iter);
 
 	/* fetch the previous row if necessary */
 	switch (imodel->priv->format) {
@@ -1929,24 +1916,30 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 		GSList *plist;
 		GSList *vlist;
 		gboolean update_model;
-		
-		g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
-		g_object_set (G_OBJECT (iter), "update_model", FALSE, NULL);
-		plist = ((GdaSet *) iter)->holders;
-		vlist = imodel->priv->cursor_values;
-		while (plist && vlist) {
-			gda_holder_set_value (GDA_HOLDER (plist->data), 
-					      (GValue *) vlist->data);
-			plist = g_slist_next (plist);
-			vlist = g_slist_next (vlist);
+		gboolean allok = TRUE;
+
+		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
+		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
+		for (plist = ((GdaSet *) iter)->holders, vlist = imodel->priv->cursor_values;
+		     plist && vlist;
+		     plist = plist->next, vlist = vlist->next) {
+			GError *lerror = NULL;
+			if (! gda_holder_set_value (GDA_HOLDER (plist->data), 
+						    (GValue *) vlist->data, &lerror)) {
+				gchar *tmp;
+				tmp = g_strdup_printf (_("Could not set iterator's value: %s"),
+						       lerror && lerror->message ? lerror->message : _("No detail"));
+				add_error (imodel, tmp);
+				g_free (tmp);
+				allok = FALSE;
+			}
 		}
 		if (plist || vlist) {
+			allok = FALSE;
 			if (plist) {
 				add_error_too_few_values (imodel);
-				while (plist) {
-					gda_holder_set_value (GDA_HOLDER (plist->data), NULL);
-					plist = g_slist_next (plist);
-				}				
+				for (; plist; plist = plist->next)
+					gda_holder_force_invalid (GDA_HOLDER (plist->data));
 			}
 			else 
 				add_error_too_many_values (imodel);
@@ -1957,8 +1950,8 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 
 		g_assert (imodel->priv->iter_row >= 0);
 		g_object_set (G_OBJECT (iter), "current-row", imodel->priv->iter_row, 
-			      "update_model", update_model, NULL);
-		return TRUE;
+			      "update-model", update_model, NULL);
+		return allok;
 	}
 	else {
 		g_object_set (G_OBJECT (iter), "current-row", -1, NULL);
